==> v5.61.tar.gz: wcs/tests/test_cards_admin_pages.py <==
# -*- coding: utf-8 -*-

import pytest

from wcs import fields
from wcs.admin.settings import UserFieldsFormDef
from wcs.carddef import CardDef
from wcs.formdef import FormDef
from wcs.qommon.http_request import HTTPRequest
from wcs.roles import Role
from wcs.wf.form import FormWorkflowStatusItem, WorkflowFormFieldsFormDef
from wcs.workflows import Workflow
from wcs.workflows import WorkflowBackofficeFieldsFormDef
from wcs.workflows import WorkflowVariablesFieldsFormDef

from utilities import get_app, login, create_temporary_pub, clean_temporary_pub
from test_admin_pages import create_superuser


def pytest_generate_tests(metafunc):
    if 'pub' in metafunc.fixturenames:
        metafunc.parametrize('pub', ['pickle', 'sql', 'pickle-templates'], indirect=True)


@pytest.fixture
def pub(request):
    pub = create_temporary_pub(
            sql_mode=bool('sql' in request.param),
            templates_mode=bool('templates' in request.param)
            )

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.cfg['language'] = {'language': 'en'}
    pub.write_cfg()

    return pub


def teardown_module(module):
    clean_temporary_pub()


def test_cards_new(pub, studio):
    CardDef.wipe()
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/cards/')
    resp = resp.click('New Card Model')
    resp.form['name'] = 'card title'
    resp = resp.form.submit()
    assert resp.location == 'http://example.net/backoffice/cards/1/'
    resp = resp.follow()
    assert '<h2>card title' in resp.text
    assert CardDef.get(1).workflow_id is None
    assert CardDef.get(1).disabled is False


def test_cards_delete(pub, studio):
    CardDef.wipe()
    carddef = CardDef()
    carddef.name = 'card title'
    carddef.fields = []
    carddef.store()
    create_superuser(pub)
    app = login(get_app(pub))

    carddata = carddef.data_class()()
    carddata.just_created()
    carddata.data = {}
    carddata.store()

    resp = app.get('http://example.net/backoffice/cards/1/')
    resp = resp.click('Delete')
    if pub.is_using_postgresql():
        assert 'Deletion is not possible as there are cards.' in resp
        carddef.data_class().wipe()
        resp = app.get('http://example.net/backoffice/cards/1/')
        resp = resp.click('Delete')

    resp = resp.form.submit('submit')
    assert CardDef.count() == 0

    carddata.remove_self()  # don't keep leftovers


def test_cards_duplicate(pub, studio):
    CardDef.wipe()
    carddef = CardDef()
    carddef.name = 'card title'
    carddef.fields = []
    carddef.store()
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('http://example.net/backoffice/cards/1/')
    resp = resp.click('Duplicate')
    assert CardDef.get(2).name == 'card title (copy)'
    assert CardDef.get(2).disabled is False


def test_card_workflow_change(pub, studio):
    Workflow.wipe()
    workflow = Workflow(name='Workflow Two')
    workflow.add_status('plop')
    workflow.store()

    CardDef.wipe()
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/cards/')
    resp = resp.click('New Card Model')
    resp.form['name'] = 'card title'
    resp = resp.form.submit()
    resp = resp.follow()
    resp = resp.click(href='workflow', index=1)
    assert resp.form['workflow_id'].options[0][2] == 'Default (cards)'
    resp = resp.form.submit('submit').follow()
    assert CardDef.select()[0].workflow_id is None

    carddata = CardDef.select()[0].data_class()()
    carddata.status = 'wf-recorded'
    carddata.store()

    resp = resp.click(href='workflow', index=1)
    resp.form['workflow_id'] = '%s' % workflow.id
    resp = resp.form.submit('submit')
    assert resp.location == 'http://example.net/backoffice/cards/1/workflow-status-remapping?new=%s' % workflow.id
    resp = resp.follow()
    resp = resp.form.submit('submit').follow()

    resp = resp.click(href='workflow', index=1)
    resp.form['workflow_id'] = ''
    resp = resp.form.submit('submit')
    assert resp.location == 'http://example.net/backoffice/cards/1/workflow-status-remapping?new=%s' % '_carddef_default'
    resp = resp.follow()
    resp = resp.form.submit('submit').follow()


def test_card_custom_view_data_source(pub, studio):
    user = create_superuser(pub)
    Role.wipe()
    role = Role(name='foobar')
    role.store()
    user.roles = [role.id]
    user.store()

    CardDef.wipe()
    carddef = CardDef()
    carddef.name = 'foo'
    carddef.fields = [
        fields.StringField(id='1', label='Test', type='string', varname='foo'),
    ]
    carddef.backoffice_submission_roles = user.roles
    carddef.workflow_roles = {'_editor': user.roles[0]}
    carddef.digest_template = '{{ form_var_foo }}'
    carddef.store()
    carddef.data_class().wipe()

    for i in range(2):
        carddata = carddef.data_class()()
        carddata.data = {'1': 'FOO %s' % i}
        carddata.just_created()
        carddata.store()

    app = login(get_app(pub))
    resp = app.get('/backoffice/data/foo/')
    resp = resp.forms['listing-settings'].submit()
    resp.forms['save-custom-view']['title'] = 'card view'
    resp = resp.forms['save-custom-view'].submit()

    resp = app.get('/backoffice/data/foo/user-card-view/')
    resp.forms['listing-settings']['filter-1'].checked = True
    resp = resp.forms['listing-settings'].submit()
    resp.forms['listing-settings']['filter-1-value'] = 'FOO 0'
    resp = resp.forms['listing-settings'].submit()
    resp.forms['save-custom-view']['visibility'] = 'datasource'
    resp = resp.forms['save-custom-view'].submit()
    assert pub.custom_view_class.count() == 1
    assert pub.custom_view_class.select()[0].visibility == 'datasource'

    FormDef.wipe()
    formdef = FormDef()
    formdef.name = 'form title'
    formdef.fields = [
        fields.ItemField(id='1', label='field', type='item', varname='foo'),
    ]
    formdef.store()

    formdef = FormDef.get_by_urlname('form-title')
    resp = app.get('/backoffice/forms/%s/fields/' % formdef.id)

    resp.forms[0]['label'] = 'foobar'
    resp.forms[0]['type'] = 'item'
    resp = resp.forms[0].submit()
    resp = resp.follow()
    formdef = FormDef.get_by_urlname('form-title')
    resp = resp.click(href='%s/' % formdef.fields[-1].id, index=0)
    assert 'carddef:foo' in [x[0] for x in resp.form['data_source$type'].options]
    assert 'carddef:foo:card-view' in [x[0] for x in resp.form['data_source$type'].options]
    assert len(CardDef.get_data_source_items('carddef:foo')) == 2
    assert len(CardDef.get_data_source_items('carddef:foo:card-view')) == 1


def test_carddef_usage(pub):
    user = create_superuser(pub)
    Role.wipe()
    role = Role(name='foobar')
    role.store()
    user.roles = [role.id]
    user.store()

    # the one used as data source
    CardDef.wipe()
    carddef = CardDef()
    carddef.name = 'foo'
    carddef.fields = [
        fields.StringField(id='1', label='Test', type='string', varname='foo'),
    ]
    carddef.workflow_roles = {'_editor': user.roles[0]}
    carddef.store()

    # another one
    carddef2 = CardDef()
    carddef2.name = 'foobar'  # url_name startswith 'foo'
    carddef2.fields = [
        fields.StringField(id='1', label='Test', type='string', varname='foo'),
    ]
    carddef2.store()

    # custom view
    app = login(get_app(pub))
    resp = app.get('/backoffice/data/foo/')
    resp = resp.forms['listing-settings'].submit()
    resp.forms['save-custom-view']['title'] = 'card view'
    resp = resp.forms['save-custom-view'].submit()

    # a formdef using the carddef as datasource
    FormDef.wipe()
    formdef = FormDef()
    formdef.name = 'test card def on data source'
    formdef.fields = [
        fields.ItemField(id='1', label='item', data_source={'type': 'carddef:foo'})
    ]
    formdef.store()
    # another using the custom view
    formdef2 = FormDef()
    formdef2.name = 'test card def on data source2'
    formdef2.fields = [
        fields.ItemField(id='1', label='item', data_source={'type': 'carddef:foo:card-view'})
    ]
    formdef2.store()
    # another, using the other carddef as datasource
    formdef3 = FormDef()
    formdef3.name = 'test card def on data source2'
    formdef3.fields = [
        fields.ItemField(id='1', label='item', data_source={'type': 'carddef:foobar'})  # startswith carddef:foo
    ]
    formdef3.store()

    # user form
    user_formdef = UserFieldsFormDef(pub)
    user_formdef.fields.append(
        fields.ItemField(id='1', type='item', label='item', data_source={'type': 'carddef:foo'}))
    user_formdef.store()

    # workflow
    Workflow.wipe()
    workflow = Workflow(name='Workflow One')
    workflow.variables_formdef = WorkflowVariablesFieldsFormDef(workflow=workflow)
    workflow.variables_formdef.fields.append(
        fields.ItemField(id='1', type='idem', label='item', data_source={'type': 'carddef:foo'}))
    workflow.backoffice_fields_formdef = WorkflowBackofficeFieldsFormDef(workflow)
    workflow.backoffice_fields_formdef.fields.append(
        fields.ItemField(id='1', type='item', label='item', data_source={'type': 'carddef:foo'}))

    baz_status = workflow.add_status(name='baz')
    display_form = FormWorkflowStatusItem()
    display_form.id = '_x'
    display_form.formdef = WorkflowFormFieldsFormDef(item=display_form)
    display_form.formdef.fields.append(
        fields.ItemField(id='1', type='item', label='item', data_source={'type': 'carddef:foo'}))
    baz_status.items.append(display_form)
    display_form.parent = baz_status

    workflow.store()

    # carddef
    carddef3 = CardDef()
    carddef3.name = 'Baz'
    carddef3.fields = [
        fields.ItemField(id='1', label='item', data_source={'type': 'carddef:foo'})
    ]
    carddef3.store()

    resp = app.get('http://example.net/backoffice/cards/1/')
    assert 'This card model is used as data source in the following forms' in resp.text
    assert '/backoffice/forms/%s/' % formdef.id in resp.text
    assert '/backoffice/forms/%s/' % formdef2.id in resp.text
    assert '/backoffice/forms/%s/' % formdef3.id not in resp.text  # no, not the good one
    assert '/backoffice/workflows/%s/backoffice-fields/fields/' % workflow.id in resp.text
    assert '/backoffice/workflows/%s/variables/fields/' % workflow.id in resp.text
    assert '/backoffice/workflows/%s/status/1/items/_x/fields/' % workflow.id in resp.text
    assert '/backoffice/settings/users/fields/' in resp.text
    assert '/backoffice/cards/%s/' % carddef3.id in resp.text

    # cleanup
    user_formdef = UserFieldsFormDef(pub)
    user_formdef.fields = []
    user_formdef.store()

==> v5.64.tar.gz: wcs/tests/test_cards_admin_pages.py <==
# -*- coding: utf-8 -*-

import pytest

from wcs import fields
from wcs.admin.settings import UserFieldsFormDef
from wcs.carddef import CardDef
from wcs.formdef import FormDef
from wcs.qommon.http_request import HTTPRequest
from wcs.roles import Role
from wcs.wf.form import FormWorkflowStatusItem, WorkflowFormFieldsFormDef
from wcs.workflows import Workflow
from wcs.workflows import WorkflowBackofficeFieldsFormDef
from wcs.workflows import WorkflowVariablesFieldsFormDef

from utilities import get_app, login, create_temporary_pub, clean_temporary_pub
from test_admin_pages import create_superuser


def pytest_generate_tests(metafunc):
    if 'pub' in metafunc.fixturenames:
        metafunc.parametrize('pub', ['pickle', 'sql', 'pickle-templates'], indirect=True)


@pytest.fixture
def pub(request):
    pub = create_temporary_pub(
            sql_mode=bool('sql' in request.param),
            templates_mode=bool('templates' in request.param)
            )

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.cfg['language'] = {'language': 'en'}
    pub.write_cfg()

    return pub


def teardown_module(module):
    clean_temporary_pub()


def test_cards_new(pub, studio):
    CardDef.wipe()
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/cards/')
    resp = resp.click('New Card Model')
    resp.form['name'] = 'card title'
    resp = resp.form.submit()
    assert resp.location == 'http://example.net/backoffice/cards/1/'
    resp = resp.follow()
    assert '<h2>card title' in resp.text
    assert CardDef.get(1).workflow_id is None
    assert CardDef.get(1).disabled is False


def test_cards_delete(pub, studio):
    CardDef.wipe()
    carddef = CardDef()
    carddef.name = 'card title'
    carddef.fields = []
    carddef.store()
    create_superuser(pub)
    app = login(get_app(pub))

    carddata = carddef.data_class()()
    carddata.just_created()
    carddata.data = {}
    carddata.store()

    resp = app.get('http://example.net/backoffice/cards/1/')
    resp = resp.click('Delete')
    if pub.is_using_postgresql():
        assert 'Deletion is not possible as there are cards.' in resp
        carddef.data_class().wipe()
        resp = app.get('http://example.net/backoffice/cards/1/')
        resp = resp.click('Delete')

    resp = resp.form.submit('submit')
    assert CardDef.count() == 0

    carddata.remove_self()  # don't keep leftovers


def test_cards_duplicate(pub, studio):
    CardDef.wipe()
    carddef = CardDef()
    carddef.name = 'card title'
    carddef.fields = []
    carddef.store()
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('http://example.net/backoffice/cards/1/')
    resp = resp.click('Duplicate')
    assert CardDef.get(2).name == 'card title (copy)'
    assert CardDef.get(2).disabled is False


def test_card_workflow_change(pub, studio):
    Workflow.wipe()
    workflow = Workflow(name='Workflow Two')
    workflow.add_status('plop')
    workflow.store()

    CardDef.wipe()
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/cards/')
    resp = resp.click('New Card Model')
    resp.form['name'] = 'card title'
    resp = resp.form.submit()
    resp = resp.follow()
    resp = resp.click(href='workflow', index=1)
    assert resp.form['workflow_id'].options[0][2] == 'Default (cards)'
    resp = resp.form.submit('submit').follow()
    assert CardDef.select()[0].workflow_id is None

    carddata = CardDef.select()[0].data_class()()
    carddata.status = 'wf-recorded'
    carddata.store()

    resp = resp.click(href='workflow', index=1)
    resp.form['workflow_id'] = '%s' % workflow.id
    resp = resp.form.submit('submit')
    assert resp.location == 'http://example.net/backoffice/cards/1/workflow-status-remapping?new=%s' % workflow.id
    resp = resp.follow()
    resp = resp.form.submit('submit').follow()

    resp = resp.click(href='workflow', index=1)
    resp.form['workflow_id'] = ''
    resp = resp.form.submit('submit')
    assert resp.location == 'http://example.net/backoffice/cards/1/workflow-status-remapping?new=%s' % '_carddef_default'
    resp = resp.follow()
    resp = resp.form.submit('submit').follow()


def test_card_custom_view_data_source(pub, studio):
    user = create_superuser(pub)
    Role.wipe()
    role = Role(name='foobar')
    role.store()
    user.roles = [role.id]
    user.store()

    CardDef.wipe()
    carddef = CardDef()
    carddef.name = 'foo'
    carddef.fields = [
        fields.StringField(id='1', label='Test', type='string', varname='foo'),
    ]
    carddef.backoffice_submission_roles = user.roles
    carddef.workflow_roles = {'_editor': user.roles[0]}
    carddef.digest_template = '{{ form_var_foo }}'
    carddef.store()
    carddef.data_class().wipe()

    for i in range(3):
        carddata = carddef.data_class()()
        carddata.just_created()
        if i == 0:
            carddata.data = {'1': 'BAR'}
        else:
            carddata.data = {'1': 'FOO'}
        if i == 1:
            carddata.jump_status('deleted')
        carddata.store()

    pub.custom_view_class.wipe()
    custom_view = pub.custom_view_class()
    custom_view.title = 'card view'
    custom_view.formdef = carddef
    custom_view.columns = {'list': [{'id': 'id'}]}
    custom_view.filters = {'filter-1': True, 'filter-1-value': 'FOO'}
    custom_view.visibility = 'datasource'
    custom_view.store()

    FormDef.wipe()
    formdef = FormDef()
    formdef.name = 'form title'
    formdef.fields = [
        fields.ItemField(id='1', label='field', type='item', varname='foo'),
    ]
    formdef.store()

    app = login(get_app(pub))
    resp = app.get('/backoffice/forms/%s/fields/1/' % formdef.id)
    assert 'carddef:foo' in [x[0] for x in resp.form['data_source$type'].options]
    assert 'carddef:foo:card-view' in [x[0] for x in resp.form['data_source$type'].options]

    assert len(CardDef.get_data_source_items('carddef:foo')) == 3
    assert len(CardDef.get_data_source_items('carddef:foo:card-view')) == 2

    custom_view.filters = {'filter-1': True, 'filter-1-value': 'FOO', 'filter-status': 'on', 'filter': 'done'}
    custom_view.store()
    assert len(CardDef.get_data_source_items('carddef:foo:card-view')) == 1


def test_carddef_usage(pub):
    user = create_superuser(pub)
    Role.wipe()
    role = Role(name='foobar')
    role.store()
    user.roles = [role.id]
    user.store()

    # the one used as data source
    CardDef.wipe()
    carddef = CardDef()
    carddef.name = 'foo'
    carddef.fields = [
        fields.StringField(id='1', label='Test', type='string', varname='foo'),
    ]
    carddef.workflow_roles = {'_editor': user.roles[0]}
    carddef.store()

    # another one
    carddef2 = CardDef()
    carddef2.name = 'foobar'  # url_name startswith 'foo'
    carddef2.fields = [
        fields.StringField(id='1', label='Test', type='string', varname='foo'),
    ]
    carddef2.store()

    # custom view
    app = login(get_app(pub))
    resp = app.get('/backoffice/data/foo/')
    resp = resp.forms['listing-settings'].submit()
    resp.forms['save-custom-view']['title'] = 'card view'
    resp = resp.forms['save-custom-view'].submit()

    # a formdef using the carddef as datasource
    FormDef.wipe()
    formdef = FormDef()
    formdef.name = 'test card def on data source'
    formdef.fields = [
        fields.ItemField(id='1', label='item', data_source={'type': 'carddef:foo'})
    ]
    formdef.store()
    # another using the custom view
    formdef2 = FormDef()
    formdef2.name = 'test card def on data source2'
    formdef2.fields = [
        fields.ItemField(id='1', label='item', data_source={'type': 'carddef:foo:card-view'})
    ]
    formdef2.store()
    # another, using the other carddef as datasource
    formdef3 = FormDef()
    formdef3.name = 'test card def on data source2'
    formdef3.fields = [
        fields.ItemField(id='1', label='item', data_source={'type': 'carddef:foobar'})  # startswith carddef:foo
    ]
    formdef3.store()

    # user form
    user_formdef = UserFieldsFormDef(pub)
    user_formdef.fields.append(
        fields.ItemField(id='1', type='item', label='item', data_source={'type': 'carddef:foo'}))
    user_formdef.store()

    # workflow
    Workflow.wipe()
    workflow = Workflow(name='Workflow One')
    workflow.variables_formdef = WorkflowVariablesFieldsFormDef(workflow=workflow)
    workflow.variables_formdef.fields.append(
        fields.ItemField(id='1', type='idem', label='item', data_source={'type': 'carddef:foo'}))
    workflow.backoffice_fields_formdef = WorkflowBackofficeFieldsFormDef(workflow)
    workflow.backoffice_fields_formdef.fields.append(
        fields.ItemField(id='1', type='item', label='item', data_source={'type': 'carddef:foo'}))

    baz_status = workflow.add_status(name='baz')
    display_form = FormWorkflowStatusItem()
    display_form.id = '_x'
    display_form.formdef = WorkflowFormFieldsFormDef(item=display_form)
    display_form.formdef.fields.append(
        fields.ItemField(id='1', type='item', label='item', data_source={'type': 'carddef:foo'}))
    baz_status.items.append(display_form)
    display_form.parent = baz_status

    workflow.store()

    # carddef
    carddef3 = CardDef()
    carddef3.name = 'Baz'
    carddef3.fields = [
        fields.ItemField(id='1', label='item', data_source={'type': 'carddef:foo'})
    ]
    carddef3.store()

    resp = app.get('http://example.net/backoffice/cards/1/')
    assert 'This card model is used as data source in the following forms' in resp.text
    assert '/backoffice/forms/%s/' % formdef.id in resp.text
    assert '/backoffice/forms/%s/' % formdef2.id in resp.text
    assert '/backoffice/forms/%s/' % formdef3.id not in resp.text  # no, not the good one
    assert '/backoffice/workflows/%s/backoffice-fields/fields/' % workflow.id in resp.text
    assert '/backoffice/workflows/%s/variables/fields/' % workflow.id in resp.text
    assert '/backoffice/workflows/%s/status/1/items/_x/fields/' % workflow.id in resp.text
    assert '/backoffice/settings/users/fields/' in resp.text
    assert '/backoffice/cards/%s/' % carddef3.id in resp.text

    # cleanup
    user_formdef = UserFieldsFormDef(pub)
    user_formdef.fields = []
    user_formdef.store()
